class End:
	from ..Basic.Common_Function import Common_Function as co_f
	from ..Basic.Common_Variable import Common_Variable as co_v
	from ..Basic.Settings import Settings as st
	import datetime as dt
	import time
	
	def main(self): #データをセーブする		
		ty = self.co_f.typing("データをセーブしますか？ 0:する 1:しない", 1, 2)
		print("~" * self.st.SEPALATE_LEN)
		if ty == 0:
			self.save()
		else:
			ty = self.co_f.typing("本当にしないでよろしいですか? 0:する 1:しない", 1, 2)
			print("~" * self.st.SEPALATE_LEN)
			if ty == 0:
				self.save
		print("=" * self.st.SEPALATE_LEN)
		return
	
	def save(self):
		from ..Save_Code.Encode import Encode
		from ..CloudSave.Upload import Upload as u
		code = Encode.encode()
		rem = 0
		ty = self.co_f.typing("クラウドセーブを利用しますか？(事前登録が必要です) 0:利用する 1:利用しない", 1, 2)
		print("~注意事項~")
		print("データは上書きされます。上書きしたくない場合は新たにアカウントを作成してください。")
		if ty == 0:
			while True:
				if self.co_v.retry != 0:
					rem = rem - self.dt.datetime.now()
					rem = int(rem.total_seconds() + 0.5)
				else:
					rem = 0
				u_name = self.co_f.typing("ユーザー名", 0)
				pas = self.co_f.typing("パスワード", 0)
				print("~" * self.st.SEPALATE_LEN)

				self.time.sleep(rem) #待機時間
				if rem != 0:
						print(f"再試行されるまで{rem}秒お待ちください。")
				res = u.upload(u_name, pas, code)
				
				if res[0]:
					print("正常に保存されました。")
					break
				else:
					print(res[1])
					ty = self.co_f.typing("再試行しますか？ 0:する 1:しない(コードは表示されます)", 1, 2)
					print("~" * self.st.SEPALATE_LEN)
					if ty == 0:
						now = self.dt.datetime.now()
						rem = now + self.dt.timedelta(seconds=self.st.RETRY)
					else:
						print("-" * self.st.SEPALATE_LEN)
						print("プレイヤー名と共に保管してください。")
						print(f"コード: {code}")
						print(f"プレイヤー名: {self.co_v.player_name}")


class Dictionary:
	def dic(): #イズモン辞典を表示
		from .Basic.Common_Function import Common_Function as co_f
		from .Basic.Common_Variable import Common_Variable as co_v
		from .Basic.Settings import Settings as st
		IZ_SET = st.IZUMON_SETTING
		RACE = st.RACE
		co_f.show("=" * st.SEPALATE_LEN, 0)
		action = co_f.typing("何をするのじゃ？ 0:Myイズモン一覧を見る 1:種族値を見る",1 , 2)
		co_f.show("=" * st.SEPALATE_LEN, 0)
		if action == 0:
			for i in range(st.IZ_NUM):
				if co_v.fellows[i] == 1:
					print(IZ_SET[i][1])
					print(" | コード: %s 種族: %s HP: %d 攻撃: %d 防御: %d 特攻: %d 特防: %d" %
					(IZ_SET[i][0], RACE[IZ_SET[i][2]], IZ_SET[i][3], IZ_SET[i][4], IZ_SET[i][5], IZ_SET[i][6], IZ_SET[i][7]))
		else:
			while True:
				iz = co_f.typing("どのイズモンの種族値を見るのじゃ？",1, st.IZ_NUM + 1) - 1 #コードと番地は1ずれている
				co_f.show("=" * st.SEPALATE_LEN, 0)
				print(IZ_SET[iz][1])
				print(" | -- コード: %s 種族: %s HP: %d 攻撃: %d 防御: %d 特攻: %d 特防: %d" %
				(IZ_SET[iz][0], RACE[IZ_SET[iz][2]], IZ_SET[iz][3], IZ_SET[iz][4], IZ_SET[iz][5], IZ_SET[iz][6], IZ_SET[iz][7]))
				co_f.show("=" * st.SEPALATE_LEN, 0)
				action = co_f.typing("どうするのじゃ？ 0:戻る 1:まだ見る", 1, 2)
				if action == 0:
					break


class Download:
	def download(u_name, pas):
		from .Common import Common
		import requests
		import json
		c = Common()
		
		res, name, sheet, row, pass_hash = c.get_values(u_name, pas)
		
		if not res:
			return False, "不正なユーザーネーム又はパスワードです。", ""
		url = "https://izmon-cloud-save-auv0edkp.an.gateway.dev/download"
		data = {
		"name": name,
		"sheet": str(sheet),
		"password" : pass_hash,
		"row" : str(row)
		}
	
		headers = {"Content-Type": "application/json"}
		code = ""; name = ""
		response = requests.post(url, data=json.dumps(data), headers=headers)
		status = response.status_code
		
		if status == '400':
			return False, "エラーが発生しました。時間を置いて再試行してください。", ""
		elif status == '403':
			return False, "ユーザーネームが一致しません。", ""
		elif status == '405':
			return False, "パスワードが一致しません。", ""
		
		response_json = response.json()
		name = response_json['name']
		code = response_json['code']

		return True, code, name
	


class Common:
	import datetime as dt
	from ..Basic.Common_Variable import Common_Variable as co_v
	def hash_sha1(self, password):
		import hashlib
		
		password_hash = str(hashlib.sha1(password.encode('utf-8')).hexdigest())
		password_1 = password_hash[:10]
		password_2 = password_hash[10:20]
		password_save = str(int(password_1, 16)).zfill(10) + str(int(password_2, 16)).zfill(10)
		password_save = password_save[:20]
		return password_save
	
	def regular(self, value : str):
		import re
		res = re.fullmatch(".*[0-9]{6}", value)
		if res == None:
			return False
		return True
	
	def get_values(self, u_name, pas):
		if not self.regular(u_name):
			return False, "", "", "", ""
		elif len(pas) > 20 or len(u_name) > 26:
			return False, "", "", "", ""
		name = u_name[:-6]
		sheet = int(u_name[-6:-4])
		row = int(u_name[-4:])
		pass_hash = self.hash_sha1(pas)
		return True, name, sheet, row, pass_hash
	
	def retry(self):
		now = self.dt.datetime.now()
		self.co_v.retry = now + self.dt.timedelta(seconds=15)


from setuptools import setup, find_packages


setup(
    name='Class',

    version='1.1.0',

    description='',
    long_description='',

    author='Yohei Akita, nabe, miso',
    author_email='akita.yohei0724@gmail.com',

    license='GPL v2.0',

    packages=find_packages()
)



class Gym:
	from .Basic.Settings import Settings as st
	from .Basic.Common_Function import Common_Function as co_f
	from .Basic.Common_Variable import Common_Variable as co_v
	from .Battle.Battle import Battle as bat
	from .Battle.Battle_Set import Battle_Set as bat_s

	hp_me = [0, 0, 0, 0, 0, 0]
	
	def create_party(self):
		party_num = []
		num = 0
		nums = []
		action = self.co_f.typing("どうするのじゃ？ 0:削除 1:追加 2:全削除", 1, 3)
		if action == 0:
			print("現在のパーティー一覧")
			print("~" * self.st.SEPALATE_LEN)
			for i in range(self.st.PARTY_NUM):
				party_num.append(self.co_v.party[i])
				if self.co_v.party[i] == 0:
					continue
				print(i, ":", self.st.IZUMON_SETTING[party_num[i]][1])
			print("~" * self.st.SEPALATE_LEN)
			kind = self.co_f.typing("どれを削除しますか？", 1, self.st.PARTY_NUM)
			party_num[kind] = 0
			self.co_v.party[kind] = 0
			for i in range(self.st.PARTY_NUM - 1):
				if party_num[i] == 0:
					party_num[i] = party_num[i + 1]
					party_num[i + 1] = 0
		elif action == 1:
			nums = []; num = 0
			print("~" * self.st.SEPALATE_LEN)
			print("手持ちのイズモン一覧")
			for i in range(self.st.IZ_NUM):
				if self.co_v.fellows[i] == 1:
					print(" | --", num, self.st.IZUMON_SETTING[i][1])
					nums.append(self.st.IZUMON_SETTING[i][0] - 1)
					num += 1
			print("~" * self.st.SEPALATE_LEN)
			action = self.co_f.typing("どれを追加するのじゃ?", 1, num)
			iz = nums[action]
			if iz in self.co_v.party:
				print("もうパーティーにおるぞ")
			else:
				self.co_v.party[self.bat_s.count_party()] = iz
				print("追加したぞ")
		elif action == 2:
			for i in range(self.st.PARTY_NUM):
				self.co_v.party[i] = 0
			print("全部削除したぞ")
		return
	
	def battle_by_code(self):
		return
	
	def battle_by_default(self):
		leadar_len = len(self.st.GYM_DEFAULT_NAME)
		print("ジムリーダー一覧")
		print("~" * self.st.SEPALATE_LEN)

		for i in range(leadar_len):
			print(f" | -- {i} :{self.st.GYM_DEFAULT_NAME[i][0]} ( {self.st.GYM_DEFAULT_NAME[i][1]} )")
		
		print("~" * self.st.SEPALATE_LEN)
		action = self.co_f.typing(f"どのジムリーダーと戦うのじゃ？( {str(leadar_len)} でやめる)", 1, leadar_len)
		if action == leadar_len:
			self.co_f.show("やめるのじゃな")
			return
		leadar = self.st.GYM_DEFAULT[action]
		gym_num = len(leadar)
		for i in range(gym_num):
			self.hp_me[i] = self.st.IZUMON_SETTING[self.co_v.party[i]][3]
		for i in range(gym_num):
			
			res = self.bat.battle(leadar[i] - 1, 1)
			if res == 0:
				self.co_f.show("またチャレンジするんじゃぞ")
				break
			elif res == 1 and i != gym_num:
				print("=" * self.st.SEPALATE_LEN)
			elif res == 1 and i == gym_num:
				self.co_f.show("チャンレンジ成功")
				self.co_f.show("よく倒したな")
				self.get_badge(action)
			elif res == 2:
				self.co_f.show("チャレンジ失敗")
				self.co_f.show("またチャレンジするんじゃぞ")
				break
		return
	
	def gym_main(self):
		print("=" * self.st.SEPALATE_LEN)
		self.co_f.show("イズモンジムにようこそ!")
		while True:
			self.co_f.show("何をするのじゃ？", 0)
			action = self.co_f.typing("0:パーティーを変える 1:ジムリーダーと戦う 2:バッジ一覧を見る 3:ジムを出る", 1, 4)
			if action == 0:
				self.create_party()
			elif action == 1:
				if self.bat_s.set_ene_level(1) < self.st.LEVEL_MAX[max(0, sum(self.co_v.badges) - 1)]:
					self.co_f.show(f"パーティーのレベルが低すぎるぞ(あと{self.st.LEVEL_MAX[max(0, sum(self.co_v.badges) - 1)] - self.bat_s.set_ene_level(1)}レベル)")
					self.co_f.show("もう少しバトルでイズモンを強くしてから来るのじゃ")
					continue
				if sum(self.co_v.party) != 0:
					self.battle_by_default()
				else:
					self.co_f.show("パーティーにイズモンがいないぞ")
					self.co_f.show("イズモンなしでは戦えんぞ")
					continue
			elif action == 2:
				self.show_badge()
			else:
				if sum(self.co_v.party) != 0:
					self.co_f.show("また来るのじゃぞ")
					break
				else:
					self.co_f.show("パーティーにイズモンがいないぞ")
			print("-" * self.st.SEPALATE_LEN)
		for i in range(self.st.PARTY_NUM):
			self.hp_me[i] = 0
		print("=" * self.st.SEPALATE_LEN)
		return
	
	def get_badge(self, leader : int):
		self.co_v.badges[leader] = 1
		self.co_f.show("バッジを獲得したぞ")
		return
	
	def show_badge(self):
		print("手持ちのバッジ一覧")
		print("~" * self.st.SEPALATE_LEN)
		for i in range(len(self.co_v.badges)):
			if self.co_v.badges[i] == 1:
				print(f"{self.st.GYM_DEFAULT_NAME[i][0]} ( {self.st.GYM_DEFAULT_NAME[i][1]}")
		print("~" * self.st.SEPALATE_LEN)
		return



class Decode:
	def decode(code):
		from ..Basic.Settings import Settings as st
		from ..Basic.Common_Variable import Common_Variable as co_v
		import hashlib
		
		code_in = code
		
		code_10 = str(int(code_in, 16))
		
		has = hashlib.sha1(co_v.player_name.encode('utf-8')).hexdigest()
		hash_10 = str(int(has, 16))
		del has
		
		if not (int(code_10[:-1]) % int(hash_10[20])) == int(code_10[-1]):
			return 1
		code_remove_digit = code_10[:-1]
		del code_10
		
		if not (code_remove_digit[-10:] == hash_10[:10]):
			return 1
		if not (code_remove_digit[0] == hash_10[11]):
			return 1
		if not (code_remove_digit[1] == hash_10[30]):
			return 1
		code_remove_hash = code_remove_digit[2:-10]
		del code_remove_digit
		
		data_fellow = code_remove_hash[:st.CODE_FELLOW_LEN]
		code_removed = code_remove_hash[st.CODE_FELLOW_LEN:]
		data_badge = code_removed[-3:]
		code_removed = code_removed[:-3]
		data_party = code_removed[-(st.PARTY_NUM * 3):]
		data_level = code_removed[:-(st.PARTY_NUM * 3)]
		
		data_fellow_2 = str(format(int(data_fellow), 'b')).zfill(st.CODE_CAP)
		data_badge_10 = str(format(int(data_badge), 'b')).zfill(st.PARTY_NUM)
		for i in range(st.CODE_CAP):
			co_v.fellows[i] = int(data_fellow_2[i])
			co_v.level[i] = int(data_level[(i * 3):(i * 3 + 3)])
			
		for i in range(st.PARTY_NUM):
			co_v.party[i] = int(data_party[(i * 3):(i * 3 + 3)])
			
		for i in range(len(co_v.badges)):
			co_v.badges[i] = int(data_badge_10[i])
		del code_remove_hash; del data_fellow; del code_removed; del data_badge
		del data_level; del data_party; del data_badge_10; del data_fellow_2
		return 0


class Battle:
	def battle(ene : int, typ : int): #typ 0:通常 1:ジム
		import random
		from .Battle_Set import Battle_Set as bat_s
		from .Battle_Other import Battle_Other
		from ..Basic.Common_Function import Common_Function as co_f
		from ..Basic.Common_Variable import Common_Variable as co_v
		from ..Basic.Settings import Settings as st
		from ..Gym import Gym

		gym = Gym(); bat_o = Battle_Other()

		data , damage_all, tech = [], [], []
		party_num = bat_s.count_party()
		
		turn = bat_o.set_turn()
		print("=" * st.SEPALATE_LEN)
		ene_data = st.IZUMON_SETTING[ene]
		if typ == 0:
			co_f.show("野生の" + ene_data[1] + "が現れた! (種族:" + st.RACE[ene_data[2]] + "）")
		elif typ == 1:
			co_f.show(ene_data[1] + "が現れた!（種族:" + st.RACE[ene_data[2]] + "）")
		ene_data = None
		me_sub, sub = bat_o.change_izmon(party_num, typ)

		data_all, damage_all, tech = bat_s.set(ene, typ)
		while True:
			data = [data_all[me_sub], data_all[st.PARTY_NUM]]
			data_all[me_sub] = data[0]
			data_all[st.PARTY_NUM] = data[1]
			print(data[turn][1], "のターン（HP:",data[turn][2], "）")
			if turn == 0:
				while True:
					ty = co_f.typing("行動を選択 0:攻撃 1:特攻（残り" +str(data[0][3])+ "回）", 1, 2)
					if data[0][3] == 0 and ty == 1:
						print("残り回数がないぞ")
						continue
					elif ty == 1:
						data[0][3] -= 1
						break
					else:
						break
			else: #敵の行動を選択
				if data[1][3] == 0:
					ty = 1
				else:
					ty = random.randint(0, 1)
					data[1][3] -= 1
					
			co_f.show(data[turn][1] + " の 攻撃", )
			co_f.show(data[turn][1] + " の " + tech[ty * 2 + turn][sub[turn]] + " !")
			print("-" * st.SEPALATE_LEN)
			#命中判定
			if tech[8 + turn + ty * 2][sub[turn]] <= random.random():
				co_f.show(data[turn - 1][1] + "は" + str(int(damage_all[ty * 2 + turn][sub[turn]])) + "のダメージを受けた!")
				data[turn - 1][2] -= int(damage_all[ty * 2 + turn][sub[turn]]) #ダメージ分減らす
				if data[turn - 1][2] <= 0:
					data[turn - 1][2] = 0
				if typ == 1:
					gym.hp_me[me_sub] = data[me_sub][2]
			else:
				if turn == 0:
					co_f.show("避けられた!")
				else:
					co_f.show("避けた!")

			co_f.show(data[abs(turn - 1)][1] + " の 残り HP :" + str(data[turn - 1][2]))
			print('-' * st.SEPALATE_LEN)

			if data[0][2] <= 0:
				party_num -= 1
				co_f.show("HPが0になってしまった。")
				if party_num == 0:
					if typ == 0:
						co_f.show(co_v.player_name + " は目の前が真っ暗になった")
						return
					else:
						return 2
				me_sub, sub = bat_o.change_izmon(party_num, typ)
				turn = bat_o.turn_change(turn)
				continue

			if data[1][2] <= 0:
				co_f.show(data[1][1] + " を倒した!")
				if typ == 0:
					bat_o.get(ene)
					bat_o.level_up(me_sub)
					break
				else:
					return 1
			if typ == 0 and turn == 1:
				action = co_f.typing("どうする? 0:続ける 1:交代する 2:逃げる", 1, 3)
				if action == 2 and random.random() >= st.ESCAPE_P:
					co_f.show("逃げられなかった")
					turn = bat_o.turn_change(turn)
					print("-" * st.SEPALATE_LEN)
					continue
				elif action == 2:
					co_f.show("逃げられた!")
					print("-" * st.SEPALATE_LEN)
					break
				elif action == 1:
					me_sub, sub = bat_o.change_izmon(party_num, typ)
					turn = bat_o.turn_change(turn)
					continue
			elif typ == 1 and turn == 0:
				action = co_f.typing("どうする？ 0:続ける 1:やめる(次挑戦するときは最初からになります) 2:交代する", 1, 3)
				if action == 0:
					continue
				elif action == 1:
					return 0
				else:
					me_sub, sub = bat_o.change_izmon(party_num, typ)
			turn = bat_o.turn_change(turn)
		return



class Common_Function:
	def show(msg : str, length = "default"): #メッセージ表示(メッセージ, 待機秒数)
		from .Common_Variable import Common_Variable as co_v
		import time
		if length == "default":
			length = co_v.show_length
		print(msg)
		time.sleep(length)
		return
	
	def typing(msg : str, ty : int, num : int = 2): #各種キーボード入力
		while True:
			if ty == 1:
				data = input(msg + " -->> ")
				if data.isdigit() and 0 <= int(data) and int(data) <= num - 1:
					data = int(data)
					break
				elif data.isdigit() == False:
					print("無効な入力じゃ")
			else:
				data = str(input(msg + " -->> "))
				if data == '':
					print("無効な入力じゃ")
					continue
				break
		return data



class Upload:
    def upload(u_name, pas, code):
        from .Common import Common
        from ..Basic.Common_Variable import Common_Variable as co_v

        import requests
        import json

        c = Common()
        res, name, sheet, row, pass_hash = c.get_values(u_name, pas)
        if not res:
            return False, "不正なユーザーネーム又はパスワードです。"

        url = "https://izmon-cloud-save-auv0edkp.an.gateway.dev/upload"
        data = {
        "name": str(name),
        "code": str(code),
        "sheet": str(sheet),
        "password" : pass_hash,
        "row" : str(row),
        "player_name" : str(co_v.player_name)
        }
    
        headers = {"Content-Type": "application/json"}
    
        response = requests.post(url, data=json.dumps(data), headers=headers)
        status = response.status_code
        if status == '200':
            return True, ""
        elif status == '400':
            return False, "エラーが発生しました。時間を置いて再試行してください。"
        elif status == '403':
            return False, "ユーザーネームが一致しません。"
        elif status == '405':
            return False, "パスワードが一致しません。"
    


class Start:
	from ..Basic.Common_Function import Common_Function as co_f
	from ..Basic.Common_Variable import Common_Variable as co_v
	from ..Basic.Settings import Settings as st
	from ..Battle.Battle_Caluculate import Battle_Caluculate
	from ..Gym import Gym
	import datetime as dt
	import time

	gym = Gym(); bat_cal = Battle_Caluculate
	
	MSG_SELECT = ["イズットモンスターの世界へ ようこそ！ わしの名前は オーキ二 みんなからは イズモン はかせと 慕われておるよ ",
	"イズットモンスター………イズモン この 世界には イズットモンスターと 呼ばれる いきもの達が いたる所に 住んでいる! ",
	"人は イズモンたちと 仲良く遊んだり 一緒に戦ったり………… 助け合いながら 暮らしているのじゃ ",
	"しかし わしらは イズモンの全てを 知っている 訳ではない イズモンの 秘密は まだまだ いっぱい ある！ ",
	"わしは それを 解き明かすために 毎日 イズモンの 研究を 続けている という わけじゃ！ ",
	"さて…… そろそろ 君の 名前を 教えてもらおう！",
	" よ イズモンを1匹も持っていないだと? しかたがない この3匹から1匹選ぶのじゃ",
	"3匹のイズモンから好きなものを選ぶのじゃ",
	"じゃな。 わかったぞ",
	" ! 準備は いいかな？ いよいよ これから きみの 物語が 始まる！ ",
	"楽しいことも 苦しいことも いっぱい きみを 待っているだろう！ 夢と 冒険と！ ",
	"イズットモンスターの 世界へ！ レッツ ゴー"]

	def set_up(self): #開始する
		from ..Save_Code.Decode import Decode
		from ..CloudSave.Download import Download as d
		rem = 0
		print("イズットモンスター(Ver", self.st.VERSION, ")")
		ty = self.co_f.typing("0:はじめから 1:続きから", 1, 2)
		self.co_f.show("=" * self.st.SEPALATE_LEN, 0)
		if ty == 0:
			self.select_izmon()
		else:
				ty = self.co_f.typing("クラウドからロードしますか？ 0:する 1:しない", 1, 2)
				if ty == 0:
					while True:
						print("~" * self.st.SEPALATE_LEN)
						if rem != 0:
							rem = rem - self.dt.datetime.now()
							rem = int(rem.total_seconds() + 0.5)
						else:
							rem = 0
						print(rem)
						u_name = self.co_f.typing("ユーザー名", 0)
						pas = self.co_f.typing("パスワード", 0)
						print("~" * self.st.SEPALATE_LEN)

						if rem != 0:
							print(f"再試行されるまで{rem}秒お待ちください。")
						self.time.sleep(rem) #待機時間

						res, code, name = d.download(u_name, pas)
						if res:
							print("正常にダウンロードされました。")
							self.co_v.player_name = name
							res = Decode.decode(code)
							if res == 1:
								print("~" * self.st.SEPALATE_LEN)
								print("無効なコードじゃ。不正はダメじゃぞ")
								continue
							else:
								print("~" * self.st.SEPALATE_LEN)
								print("データを読み込みました。")
								break
						else:
							print(code)
							print("~" * self.st.SEPALATE_LEN)
							ty = self.co_f.typing("再試行しますか？ 0:する 1:しない(手動でロードできます)", 1, 2)
							print("~" * self.st.SEPALATE_LEN)
							if ty == 0:
								now = self.dt.datetime.now()
								rem = now + self.dt.timedelta(seconds=self.st.RETRY)
								continue
							else:
								self.manual()
				else:
					self.manual()
					
	def select_izmon(self): #開始時のイズモンを選択
		for i in range(6):
			self.co_f.show(self.MSG_SELECT[i])
		player = self.co_f.typing("名前", 0)
		self.co_v.player_name = player
		self.co_f.show(player + self.MSG_SELECT[6])
		self.co_f.show(self.MSG_SELECT[7])
		poc = self.co_f.typing("0:バニージャンプ 1:テクノドラゴン 2:ナイトライダー", 1, 3)
		if poc == 0:
			code = 19 - 1
		elif poc == 1:
			code = 33 - 1
		else:
			code = 36 - 1
		self.co_v.fellows[code] = 1
		self.co_v.party[0] = code
		self.gym.hp_me[0] = self.bat_cal.set_hp(code, 1)
		self.co_f.show(self.st.IZUMON_SETTING[code][1] + self.MSG_SELECT[8])
		self.co_f.show("では " + player + self.MSG_SELECT[9])
		self.co_f.show(self.MSG_SELECT[10])
		self.co_f.show(self.MSG_SELECT[11])
		self.co_f.show("=" * self.st.SEPALATE_LEN, 5)
		return

	def manual(self):
		from ..Save_Code.Decode import Decode
		while True:
			player = self.co_f.typing("君の名前を教えるのじゃ（セーブデータコードを持っているならその時の名前を教えるのじゃ）：", 0)
			self.co_v.player_name = player
			play_type = self.co_f.typing(player + " よ、セーブデータコードは持っておるか？ 0:ある 1:ない", 1, 2)
			if play_type == 0:
				print("=" * self.st.SEPALATE_LEN)
				code = self.co_f.typing("セーブデータコード：",0)
				print("=" * self.st.SEPALATE_LEN)
				res = Decode.decode(code)
				if res == 1:
					print("無効なコードじゃ。不正はダメじゃぞ")
				else:
					print("データを読み込みました。")
					break
			else:
				self.co_f.show("コードがないのなら初めから始めるぞ")
				self.co_f.show("=" * self.st.SEPALATE_LEN, 0)
				self.select_izmon()
				break
		return


from setuptools import setup, find_packages
from codecs import open
from os import path
package_name = "Izmon"
root_dir = path.abspath(path.dirname(__file__))
with open('README.md', encoding='utf-8') as f:
    long_description = f.read()
setup(
    name=package_name,
    version='1.1.2',
    description='You can play Izmon with this libraly.',
    long_description=long_description,
    requires=["requests"],
    long_description_content_type='text/markdown',
    url='https://github.com/akita0724',
    author='Yohei Akita, nabe, miso',
    author_email='akita.yohei0724@gmail.com',
    license='GPL v2.0',
    keywords='Izmon',
    packages=find_packages(),
    classifiers=[
    'Programming Language :: Python :: 3 :: Only',
    "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
    "Operating System :: OS Independent"],
    python_requires='>=3.7'
)



class Encode:
	def encode():
		from ..Basic.Settings import Settings as st
		from ..Basic.Common_Variable import Common_Variable as co_v
		import hashlib
		
		data_fellow = ''; data_level = ''; data_party = ''; data_badge = ''; hash_name = ''
		
		hash_name = hashlib.sha1(co_v.player_name.encode('utf-8')).hexdigest()
		hash_10 = str(int(hash_name, 16))
		
		for i in range(st.CODE_CAP):
			if i > st.IZ_NUM:
				data_fellow = data_fellow + '0' * (st.CODE_CAP - st.IZ_NUM - 1)
				data_level += '000' * (st.CODE_CAP - st.IZ_NUM - 1)
				break
			data_fellow += str(co_v.fellows[i])
			data_level += str(co_v.level[i]).zfill(3)
		data_fellow_10 = hash_10[30] + str(int(data_fellow, 2)).zfill(st.CODE_FELLOW_LEN)
		
		for i in range(st.PARTY_NUM):
			data_party += str(co_v.party[i]).zfill(3)
		
		for i in range(len(co_v.badges)):
			data_badge += str(co_v.badges[i])
		data_badge_10 = str(int(data_badge, 2)).zfill(3)
		
		
		data = str(hash_10[11] + data_fellow_10 + data_level + data_party + data_badge_10 + hash_10[:10])
		
		check_digit = str(int(data) % int(hash_10[20]))
		
		data += check_digit
		
		data = int(data)
		
		data_16 = format(data, 'x')
		
		return data_16